"""Add data column

Revision ID: ccf1b72fd667
Revises: 697a95c78c24
Create Date: 2022-11-29 11:52:30.267116

"""
from alembic import op
import sqlalchemy as sa
import json
import logging
from backend.models import DataColumn, Card
from backend.models.data_column import DataColumnLabelType
from backend.utils.misc import build_type
from sqlalchemy import inspect


# revision identifiers, used by Alembic.
revision = "ccf1b72fd667"
down_revision = "697a95c78c24"
branch_labels = None
depends_on = None


def upgrade():
    conn = op.get_bind()
    current_dialect = conn.dialect.name
    inspector = inspect(conn)
    table_names = inspector.get_table_names()

    label_type = build_type(DataColumnLabelType, conn=conn, current_dialect=current_dialect)

    if not DataColumn.__tablename__ in table_names:
        data_column_table = op.create_table(
            DataColumn.__tablename__,
            sa.Column("card_id", sa.Integer(), nullable=False),
            sa.Column("name", sa.String(length=64), nullable=False),
            sa.Column("label", sa.String(length=64), nullable=True),
            sa.Column("label_type", label_type, nullable=False),
            sa.Column("priority", sa.Integer(), nullable=False),
            sa.ForeignKeyConstraint(["card_id"], [f"{Card.__tablename__}.id"], ondelete="CASCADE"),
            sa.PrimaryKeyConstraint("card_id", "name"),
            keep_existing=True,
        )

    card_table = sa.Table(Card.__tablename__, sa.MetaData(), autoload=True, autoload_with=conn, resolve_fks=False)

    all_cards = conn.execute(card_table.select()).mappings().fetchall()

    for card in all_cards:
        if card["data_columns"]:
            new_data_columns = [
                {"name": dc, "label": dc, "label_type": "CONSTANT", "priority": i, "card_id": card["id"]}
                for i, dc in enumerate(json.loads(card["data_columns"]))
            ]

            conn.execute(data_column_table.insert().values(new_data_columns))


def downgrade():
    conn = op.get_bind()

    data_column_table = sa.Table(DataColumn.__tablename__, sa.MetaData(), autoload=True, autoload_with=op.get_bind())
    card_table = sa.Table(Card.__tablename__, sa.MetaData(), autoload=True, autoload_with=op.get_bind())

    all_data_columns = conn.execute(data_column_table.select()).mappings().fetchall()
    card_ids = set(map(lambda dc: dc["card_id"], all_data_columns))
    all_data_columns_per_card_id = [
        [card_id, [dc["name"] for dc in all_data_columns if dc["card_id"] == card_id]] for card_id in card_ids
    ]

    for card_id, card_data_columns in all_data_columns_per_card_id:
        conn.execute(
            card_table.update().where(card_table.c.id == card_id).values(data_columns=json.dumps(card_data_columns))
        )

    op.drop_table(DataColumn.__tablename__)
    # ### end Alembic commands ###
