from . import db, ModelBase, View
import enum
from sqlalchemy import Enum
import backend.models.keyword as keyword_model
from sqlalchemy.ext.hybrid import hybrid_property
import json
from backend.utils.misc import NpEncoder


class MultipleMatchesMode(str, enum.Enum):
    KEEP_FIRST = "KEEP_FIRST"
    SECONDARY_COLUMN = "SECONDARY_COLUMN"


DEFAULT_KEYWORD = {"primary": ""}


class Connection(ModelBase):
    __tablename_base__ = "connection"

    id = db.Column(db.Integer, primary_key=True, autoincrement=True)
    view_id = db.Column(db.Integer, db.ForeignKey(View.id, ondelete="CASCADE"), nullable=False)

    dataset_name = db.Column(db.Text, nullable=True)
    keywords_column_name = db.Column(db.Text, nullable=True)
    multiple_matches_mode = db.Column(Enum(MultipleMatchesMode), nullable=True, default=MultipleMatchesMode.KEEP_FIRST)
    secondary_column_name = db.Column(db.Text, nullable=True)
    test_keyword_dict = db.Column(db.Text, default=json.dumps(DEFAULT_KEYWORD))

    @hybrid_property
    def test_keyword(self):
        return keyword_model.Keyword(**json.loads(self.test_keyword_dict))

    @test_keyword.setter
    def test_keyword(self, value):
        self.test_keyword_dict = json.dumps(vars(value), cls=NpEncoder)
