from flask import Blueprint, request, jsonify

from backend.controllers import url_rule_ctrl
from backend.schemas import UrlRuleSchema
from backend.utils.misc import parse_req

url_rule = Blueprint("url_rule", __name__, url_prefix="/url-rule")


@url_rule.route("/create", methods=["POST"])
def create():
    req_data = parse_req(
        request=request,
        required_fields=["view_id", "mode", "matching_type", "value", "priority"],
        optional_fields=["css_selectors"],
    )
    created_rule = url_rule_ctrl.create(**req_data)
    return jsonify({"url_rule": UrlRuleSchema().dump(created_rule)})


@url_rule.route("/<url_rule_id>", methods=["PUT"])
def update(url_rule_id):
    req_data = parse_req(
        request=request,
        optional_fields=["mode", "matching_type", "value", "priority", "css_selectors"],
        useless_fields=["view_id", "id", "dss_user_id"],
    )
    updated_url_rule = url_rule_ctrl.update(url_rule_id=url_rule_id, **req_data)
    return jsonify({"url_rule": UrlRuleSchema().dump(updated_url_rule)})


@url_rule.route("/<url_rule_id>", methods=["DELETE"])
def delete(url_rule_id):
    has_been_deleted = url_rule_ctrl.delete(url_rule_id)
    return jsonify({"message": f"URL Rule {has_been_deleted} deleted."})


@url_rule.route("/<url_rule_id>/priority", methods=["PUT"])
def update_priority(url_rule_id):
    req_data = parse_req(request=request, required_fields=["old_index", "new_index"])
    _ = url_rule_ctrl.update_priority(url_rule_id=url_rule_id, **req_data)
    return jsonify({"msg": "Priority updated."})
